@@ -1,8 +1,8 @@
 mod builder;
 mod instruction;
 mod module;
-mod types;
-mod values;
+pub mod types;
+pub mod values;
 
 pub use instruction::*;
 pub use module::*;
