@@ -1,4 +1,3 @@
-use crate::model::builder::ModuleBuilder;
 use crate::model::types::*;
 use crate::model::{Expression, Name};
 use std::mem::discriminant;
@@ -23,6 +22,7 @@ pub struct Module {
     start: Option<Start>,
     imports: Vec<Import>,
     exports: Vec<Export>,
+    customs: Vec<Custom>,
 }
 
 impl Module {
@@ -39,6 +39,7 @@ impl Module {
             start: None,
             imports: vec![],
             exports: vec![],
+            customs: vec![],
         }
     }
 
@@ -193,6 +194,47 @@ pub type DataIndex = usize;
 pub type LocalIndex = usize;
 pub type LabelIndex = usize;
 
+/// Custom sections have the id 0.
+/// They are intended to be used for debugging information or third-party extensions,
+/// and are ignored by the WebAssembly semantics. Their contents consist of a name further
+/// identifying the custom section, followed by an uninterpreted sequence of bytes for custom use.
+///
+/// See https://webassembly.github.io/spec/core/binary/modules.html#binary-customsec
+///
+/// # Examples
+/// ```rust
+/// use wasm_ast::{Custom, Name};
+///
+/// let name = "version";
+/// let version = b"1.0.0";
+/// let custom = Custom::new(name.into(), version.to_vec());
+///
+/// assert_eq!(custom.name(), &Name::new(name.to_string()));
+/// assert_eq!(custom.bytes(), &version[..]);
+/// ```
+#[derive(Clone, Debug, PartialEq)]
+pub struct Custom {
+    name: Name,
+    bytes: Vec<u8>,
+}
+
+impl Custom {
+    /// Creates a new instance of a custom section.
+    pub fn new(name: Name, bytes: Vec<u8>) -> Self {
+        Custom { name, bytes }
+    }
+
+    /// The name of the custom section.
+    pub fn name(&self) -> &Name {
+        &self.name
+    }
+
+    /// The contents of the custom section.
+    pub fn bytes(&self) -> &[u8] {
+        &self.bytes
+    }
+}
+
 /// The 𝗍𝗒𝗉𝖾 of a function declares its signature by reference to a type defined in the module.
 /// The parameters of the function are referenced through 0-based local indices in the function’s body; they are mutable.
 /// The 𝗅𝗈𝖼𝖺𝗅𝗌 declare a vector of mutable local variables and their types.
@@ -201,6 +243,23 @@ pub type LabelIndex = usize;
 /// The 𝖻𝗈𝖽𝗒 is an instruction sequence that upon termination must produce a stack matching the function type’s result type.
 ///
 /// See https://webassembly.github.io/spec/core/syntax/modules.html#functions
+///
+/// # Examples
+/// ```rust
+/// use wasm_ast::{Function, TypeIndex, Expression, ResultType, ValueType, NumericInstruction, NumberType};
+///
+/// let locals: ResultType = vec![ValueType::I32, ValueType::F32].into();
+/// let body: Expression = vec![
+///     32u32.into(),
+///     2u32.into(),
+///     NumericInstruction::Multiply(NumberType::I32).into()
+/// ].into();
+/// let function = Function::new(0, locals.clone(), body.clone());
+///
+/// assert_eq!(function.kind(), 0);
+/// assert_eq!(function.locals(), &locals);
+/// assert_eq!(function.body(), &body);
+/// ```
 #[derive(Clone, Debug, PartialEq)]
 pub struct Function {
     kind: TypeIndex,
@@ -213,14 +272,17 @@ impl Function {
         Function { kind, locals, body }
     }
 
+    /// The index of the type definition for this `Function`.
     pub fn kind(&self) -> TypeIndex {
         self.kind
     }
 
+    /// The types of the locals of this `Function`.
     pub fn locals(&self) -> &ResultType {
         &self.locals
     }
 
+    /// The code for this `Function`.
     pub fn body(&self) -> &Expression {
         &self.body
     }
@@ -234,21 +296,41 @@ impl Function {
 /// Most constructs implicitly reference table index 0.
 ///
 /// See https://webassembly.github.io/spec/core/syntax/modules.html#tables
+///
+/// # Examples
+/// ```rust
+/// use wasm_ast::{Table, TableType, Limit, ReferenceType};
+///
+/// let limit = Limit::bounded(1, 2);
+/// let kind = TableType::new(limit, ReferenceType::Function);
+/// let table = Table::new(kind);
+///
+/// assert_eq!(table, kind.into());
+/// assert_eq!(table.kind(), &kind);
+/// ```
 #[derive(Copy, Clone, Debug, Eq, PartialEq)]
 pub struct Table {
     kind: TableType,
 }
 
 impl Table {
+    /// Creates a new instance of a `Table`.
     pub fn new(kind: TableType) -> Self {
         Table { kind }
     }
 
+    /// The type descriptor of this `Table`.
     pub fn kind(&self) -> &TableType {
         &self.kind
     }
 }
 
+impl From<TableType> for Table {
+    fn from(kind: TableType) -> Self {
+        Table { kind }
+    }
+}
+
 /// A memory is a vector of raw uninterpreted bytes.
 /// The 𝗆𝗂𝗇 size in the limits of the memory type specifies the initial size of that memory,
 /// while its 𝗆𝖺𝗑, if present, restricts the size to which it can grow later. Both are in units of page size.
@@ -258,21 +340,45 @@ impl Table {
 /// Most constructs implicitly reference memory index 0.
 ///
 /// See https://webassembly.github.io/spec/core/syntax/modules.html#memories
+///
+/// # Examples
+/// ```rust
+/// use wasm_ast::{Memory, MemoryType, Limit};
+///
+/// let limit = Limit::bounded(1, 2);
+/// let kind = MemoryType::new(limit);
+/// let memory = Memory::new(kind);
+///
+/// assert_eq!(memory, kind.into());
+/// assert_eq!(memory, limit.into());
+/// assert_eq!(memory.kind(), &kind);
+/// ```
 #[derive(Copy, Clone, Debug, Eq, PartialEq)]
 pub struct Memory {
     kind: MemoryType,
 }
 
 impl Memory {
+    /// Creates a new `Memory` of the given type.
     pub fn new(kind: MemoryType) -> Self {
         Memory { kind }
     }
 
+    /// The type definition for this memory.
     pub fn kind(&self) -> &MemoryType {
         &self.kind
     }
 }
 
+impl<T> From<T> for Memory
+where
+    T: Into<MemoryType>,
+{
+    fn from(kind: T) -> Self {
+        Memory { kind: kind.into() }
+    }
+}
+
 /// Each global stores a single value of the given global type.
 /// Its 𝗍𝗒𝗉𝖾 also specifies whether a global is immutable or mutable.
 /// Moreover, each global is initialized with an 𝗂𝗇𝗂𝗍 value given by a constant initializer expression.
@@ -280,6 +386,33 @@ impl Memory {
 /// starting with the smallest index not referencing a global import.
 ///
 /// See https://webassembly.github.io/spec/core/syntax/modules.html#globals
+///
+/// # Examples
+/// ## Immutable
+/// ```rust
+/// use wasm_ast::{Global, GlobalType, ValueType, Expression};
+///
+/// let kind = GlobalType::immutable(ValueType::I64);
+/// let initializer: Expression = vec![0i64.into()].into();
+/// let global = Global::immutable(ValueType::I64, initializer.clone());
+///
+/// assert_eq!(global, Global::new(kind.clone(), initializer.clone()));
+/// assert_eq!(global.kind(), &kind);
+/// assert_eq!(global.initializer(), &initializer);
+/// ```
+///
+/// ## Mutable
+/// ```rust
+/// use wasm_ast::{Global, GlobalType, ValueType, Expression};
+///
+/// let kind = GlobalType::mutable(ValueType::I64);
+/// let initializer: Expression = vec![0i64.into()].into();
+/// let global = Global::mutable(ValueType::I64, initializer.clone());
+///
+/// assert_eq!(global, Global::new(kind.clone(), initializer.clone()));
+/// assert_eq!(global.kind(), &kind);
+/// assert_eq!(global.initializer(), &initializer);
+/// ```
 #[derive(Clone, Debug, PartialEq)]
 pub struct Global {
     kind: GlobalType,
@@ -287,14 +420,33 @@ pub struct Global {
 }
 
 impl Global {
+    /// Creates a new `Global` with the given type and initializer.
     pub fn new(kind: GlobalType, initializer: Expression) -> Self {
         Global { kind, initializer }
     }
 
+    /// Creates a new `Global` for a mutable global variable.
+    pub fn mutable(kind: ValueType, initializer: Expression) -> Self {
+        Global {
+            kind: GlobalType::mutable(kind),
+            initializer,
+        }
+    }
+
+    /// Creates a new `Global` for an immutable global variable.
+    pub fn immutable(kind: ValueType, initializer: Expression) -> Self {
+        Global {
+            kind: GlobalType::immutable(kind),
+            initializer,
+        }
+    }
+
+    /// The type of this `Global`.
     pub fn kind(&self) -> &GlobalType {
         &self.kind
     }
 
+    /// The expression to initialize this `Global` with.
     pub fn initializer(&self) -> &Expression {
         &self.initializer
     }
@@ -313,6 +465,59 @@ impl Global {
 /// Element segments are referenced through element indices.
 ///
 /// See https://webassembly.github.io/spec/core/syntax/modules.html#element-segments
+///
+/// # Examples
+/// ## Active
+/// ```rust
+/// use wasm_ast::{Element, ElementInitializer, ElementMode, TableIndex, FunctionIndex, Expression, ReferenceType};
+///
+/// let offset: Expression = vec![0i32.into()].into();
+/// let initializer: ElementInitializer = vec![0].into();
+/// let element = Element::active(0, offset.clone(), ReferenceType::Function, initializer.clone());
+///
+/// assert_eq!(element, Element::new(
+///     ReferenceType::Function,
+///     ElementMode::Active(0, offset.clone()),
+///     initializer.clone()
+/// ));
+/// assert_eq!(element.kind(), ReferenceType::Function);
+/// assert_eq!(element.mode(), &ElementMode::Active(0, offset.clone()));
+/// assert_eq!(element.initializers(), &initializer);
+/// ```
+///
+/// ## Passive
+/// ```rust
+/// use wasm_ast::{Element, ElementInitializer, ElementMode, TableIndex, Expression, ReferenceType, NumericInstruction};
+///
+/// let initializer: ElementInitializer = vec![Expression::from(vec![2i32.into()])].into();
+/// let element = Element::passive(ReferenceType::External, initializer.clone());
+///
+/// assert_eq!(element, Element::new(
+///     ReferenceType::External,
+///     ElementMode::Passive,
+///     initializer.clone()
+/// ));
+/// assert_eq!(element.kind(), ReferenceType::External);
+/// assert_eq!(element.mode(), &ElementMode::Passive);
+/// assert_eq!(element.initializers(), &initializer);
+/// ```
+///
+/// ## Declarative
+/// ```rust
+/// use wasm_ast::{Element, ElementInitializer, ElementMode, TableIndex, Expression, ReferenceType, NumericInstruction};
+///
+/// let initializer: ElementInitializer = vec![Expression::from(vec![2i32.into()])].into();
+/// let element = Element::declarative(ReferenceType::External, initializer.clone());
+///
+/// assert_eq!(element, Element::new(
+///     ReferenceType::External,
+///     ElementMode::Declarative,
+///     initializer.clone()
+/// ));
+/// assert_eq!(element.kind(), ReferenceType::External);
+/// assert_eq!(element.mode(), &ElementMode::Declarative);
+/// assert_eq!(element.initializers(), &initializer);
+/// ````
 #[derive(Clone, Debug, PartialEq)]
 pub struct Element {
     kind: ReferenceType,
@@ -321,6 +526,7 @@ pub struct Element {
 }
 
 impl Element {
+    /// Creates a new instance of an element segment.
     pub fn new(kind: ReferenceType, mode: ElementMode, initializers: ElementInitializer) -> Self {
         Element {
             kind,
@@ -329,14 +535,49 @@ impl Element {
         }
     }
 
-    pub fn kind(&self) -> &ReferenceType {
-        &self.kind
+    /// Creates a passive element segment.
+    pub fn passive(kind: ReferenceType, initializers: ElementInitializer) -> Self {
+        Element {
+            kind,
+            mode: ElementMode::Passive,
+            initializers,
+        }
     }
 
+    /// Creates an active element segment.
+    pub fn active(
+        table: TableIndex,
+        offset: Expression,
+        kind: ReferenceType,
+        initializers: ElementInitializer,
+    ) -> Self {
+        Element {
+            kind,
+            mode: ElementMode::Active(table, offset),
+            initializers,
+        }
+    }
+
+    /// Creates a declarative element segment.
+    pub fn declarative(kind: ReferenceType, initializers: ElementInitializer) -> Self {
+        Element {
+            kind,
+            mode: ElementMode::Declarative,
+            initializers,
+        }
+    }
+
+    /// The reference type of the element segment.
+    pub fn kind(&self) -> ReferenceType {
+        self.kind
+    }
+
+    /// The initializer for the element segment.
     pub fn initializers(&self) -> &ElementInitializer {
         &self.initializers
     }
 
+    /// The mode of the element segment.
     pub fn mode(&self) -> &ElementMode {
         &self.mode
     }
@@ -351,6 +592,18 @@ pub enum ElementInitializer {
     FunctionIndex(Vec<FunctionIndex>),
 }
 
+impl From<Vec<Expression>> for ElementInitializer {
+    fn from(expressions: Vec<Expression>) -> Self {
+        ElementInitializer::Expression(expressions)
+    }
+}
+
+impl From<Vec<FunctionIndex>> for ElementInitializer {
+    fn from(indices: Vec<FunctionIndex>) -> Self {
+        ElementInitializer::FunctionIndex(indices)
+    }
+}
+
 /// Element segments have a mode that identifies them as either passive, active, or declarative.
 #[derive(Clone, Debug, PartialEq)]
 pub enum ElementMode {
@@ -374,6 +627,35 @@ pub enum ElementMode {
 /// Data segments are referenced through data indices.
 ///
 /// See https://webassembly.github.io/spec/core/syntax/modules.html#data-segments
+///
+/// # Examples
+/// ## Passive
+/// ```rust
+/// use wasm_ast::{Data, DataMode};
+///
+/// let initializer = vec![42];
+/// let data = Data::passive(initializer.clone());
+///
+/// assert_eq!(data, Data::new(DataMode::Passive, initializer.clone()));
+/// assert_eq!(data, initializer.into());
+/// assert_eq!(data.mode(), &DataMode::Passive);
+/// assert_eq!(data.len(), 1);
+/// assert_eq!(data.is_empty(), false);
+/// ```
+///
+/// ## Active
+/// ```rust
+/// use wasm_ast::{Data, DataMode, MemoryIndex, Expression};
+///
+/// let initializer = vec![42];
+/// let offset: Expression = vec![1u32.into()].into();
+/// let data = Data::active(0, offset.clone(), initializer.clone());
+///
+/// assert_eq!(data, Data::new(DataMode::Active(0, offset.clone()), initializer.clone()));
+/// assert_eq!(data.mode(), &DataMode::Active(0, offset));
+/// assert_eq!(data.len(), 1);
+/// assert_eq!(data.is_empty(), false);
+/// ```
 #[derive(Clone, Debug, PartialEq)]
 pub struct Data {
     mode: DataMode,
@@ -381,27 +663,57 @@ pub struct Data {
 }
 
 impl Data {
+    /// Creates an instance of a data segment.
     pub fn new(mode: DataMode, initializer: Vec<u8>) -> Self {
         Data { mode, initializer }
     }
 
+    /// Creates an instance of a passive data segment.
+    pub fn passive(initializer: Vec<u8>) -> Self {
+        Data {
+            mode: DataMode::Passive,
+            initializer,
+        }
+    }
+
+    /// Creates an instance of an active data segment.
+    pub fn active(memory: MemoryIndex, offset: Expression, initializer: Vec<u8>) -> Self {
+        Data {
+            mode: DataMode::Active(memory, offset),
+            initializer,
+        }
+    }
+
+    /// The mode of the data segment.
     pub fn mode(&self) -> &DataMode {
         &self.mode
     }
 
+    /// The data to initialize the segment with.
     pub fn initializer(&self) -> &[u8] {
         &self.initializer
     }
 
+    /// The number of bytes in the data segment initializer.
     pub fn len(&self) -> usize {
         self.initializer.len()
     }
 
+    /// True if the data segment's initializer's length is zero, false otherwise.
     pub fn is_empty(&self) -> bool {
         self.initializer.is_empty()
     }
 }
 
+impl From<Vec<u8>> for Data {
+    fn from(initializer: Vec<u8>) -> Self {
+        Data {
+            mode: DataMode::Passive,
+            initializer,
+        }
+    }
+}
+
 /// Like element segments, data segments have a mode that identifies them as either passive or active.
 #[derive(Clone, Debug, PartialEq)]
 pub enum DataMode {
@@ -418,16 +730,25 @@ pub enum DataMode {
 /// start::={𝖿𝗎𝗇𝖼 funcidx}
 ///
 /// See https://webassembly.github.io/spec/core/syntax/modules.html#start-function
+///
+/// # Examples
+/// ```rust
+/// use wasm_ast::Start;
+///
+/// assert_eq!(Start::new(0).function(), 0);
+/// ```
 #[derive(Copy, Clone, Debug, Eq, PartialEq)]
 pub struct Start {
     function: FunctionIndex,
 }
 
 impl Start {
+    /// Creates a new instance of `Start` referencing the given function.
     pub fn new(function: FunctionIndex) -> Self {
         Start { function }
     }
 
+    /// The index of the function to run at module instantiation.
     pub fn function(&self) -> FunctionIndex {
         self.function
     }
@@ -440,6 +761,59 @@ impl Start {
 /// which are referenced through a respective descriptor.
 ///
 /// See https://webassembly.github.io/spec/core/syntax/modules.html#exports
+///
+/// # Examples
+/// ## Table
+/// ```rust
+/// use wasm_ast::{Export, ExportDescription, Name};
+///
+/// let name = "functions";
+/// let description = ExportDescription::Table(0);
+/// let export = Export::new(name.into(), description.clone());
+///
+/// assert_eq!(export, Export::table(name.into(), 0));
+/// assert_eq!(export.name(), &Name::new(String::from(name)));
+/// assert_eq!(export.description(), &description);
+/// ```
+///
+/// ## Memory
+/// ```rust
+/// use wasm_ast::{Export, ExportDescription, Name};
+///
+/// let name = "io";
+/// let description = ExportDescription::Memory(1);
+/// let export = Export::new(name.into(), description.clone());
+///
+/// assert_eq!(export, Export::memory(name.into(), 1));
+/// assert_eq!(export.name(), &Name::new(String::from(name)));
+/// assert_eq!(export.description(), &description);
+/// ```
+///
+/// ## Function
+/// ```rust
+/// use wasm_ast::{Export, ExportDescription, Name};
+///
+/// let name = "print";
+/// let description = ExportDescription::Function(42);
+/// let export = Export::new(name.into(), description.clone());
+///
+/// assert_eq!(export, Export::function(name.into(), 42));
+/// assert_eq!(export.name(), &Name::new(String::from(name)));
+/// assert_eq!(export.description(), &description);
+/// ```
+///
+/// ## Global
+/// ```rust
+/// use wasm_ast::{Export, ExportDescription, Name};
+///
+/// let name = "functions";
+/// let description = ExportDescription::Global(2);
+/// let export = Export::new(name.into(), description.clone());
+///
+/// assert_eq!(export, Export::global(name.into(), 2));
+/// assert_eq!(export.name(), &Name::new(String::from(name)));
+/// assert_eq!(export.description(), &description);
+/// ```
 #[derive(Clone, Debug, Eq, PartialEq)]
 pub struct Export {
     name: Name,
@@ -447,14 +821,49 @@ pub struct Export {
 }
 
 impl Export {
+    /// Create a new instance of an `Export` with the given name and description.
     pub fn new(name: Name, description: ExportDescription) -> Self {
         Export { name, description }
     }
 
+    /// Create a new instance of an `Export` with the given name and description for a table.
+    pub fn table(name: Name, table: TableIndex) -> Self {
+        Export {
+            name,
+            description: ExportDescription::Table(table),
+        }
+    }
+
+    /// Create a new instance of an `Export` with the given name and description for a memory.
+    pub fn memory(name: Name, memory: MemoryIndex) -> Self {
+        Export {
+            name,
+            description: ExportDescription::Memory(memory),
+        }
+    }
+
+    /// Create a new instance of an `Export` with the given name and description for a function.
+    pub fn function(name: Name, function: FunctionIndex) -> Self {
+        Export {
+            name,
+            description: ExportDescription::Function(function),
+        }
+    }
+
+    /// Create a new instance of an `Export` with the given name and description for a global.
+    pub fn global(name: Name, global: GlobalIndex) -> Self {
+        Export {
+            name,
+            description: ExportDescription::Global(global),
+        }
+    }
+
+    /// The name of the export.
     pub fn name(&self) -> &Name {
         &self.name
     }
 
+    /// The description of the table.
     pub fn description(&self) -> &ExportDescription {
         &self.description
     }
@@ -480,6 +889,70 @@ pub enum ExportDescription {
 /// definition contained in the module itself.
 ///
 /// See https://webassembly.github.io/spec/core/syntax/modules.html#imports
+///
+/// # Examples
+/// ## Table
+/// ```rust
+/// use wasm_ast::{Import, ImportDescription, Name, TableType, Limit, ReferenceType};
+///
+/// let module = "system";
+/// let name = "functions";
+/// let kind = TableType::new(Limit::unbounded(1), ReferenceType::Function);
+/// let description = ImportDescription::Table(kind.clone());
+/// let import = Import::new(module.into(), name.into(), description.clone());
+///
+/// assert_eq!(import, Import::table(module.into(), name.into(), kind));
+/// assert_eq!(import.module(), &Name::new(String::from(module)));
+/// assert_eq!(import.name(), &Name::new(String::from(name)));
+/// assert_eq!(import.description(), &description);
+/// ```
+///
+/// ## Memory
+/// ```rust
+/// use wasm_ast::{Import, ImportDescription, Name, MemoryType, Limit};
+///
+/// let module = "system";
+/// let name = "io";
+/// let kind = Limit::bounded(1, 2).into();
+/// let description = ImportDescription::Memory(kind);
+/// let import = Import::new(module.into(), name.into(), description.clone());
+///
+/// assert_eq!(import, Import::memory(module.into(), name.into(), kind));
+/// assert_eq!(import.module(), &Name::new(String::from(module)));
+/// assert_eq!(import.name(), &Name::new(String::from(name)));
+/// assert_eq!(import.description(), &description);
+/// ```
+///
+/// ## Function
+/// ```rust
+/// use wasm_ast::{Import, ImportDescription, Name};
+///
+/// let module = "system";
+/// let name = "print";
+/// let description = ImportDescription::Function(42);
+/// let import = Import::new(module.into(), name.into(), description.clone());
+///
+/// assert_eq!(import, Import::function(module.into(), name.into(), 42));
+/// assert_eq!(import.module(), &Name::new(String::from(module)));
+/// assert_eq!(import.name(), &Name::new(String::from(name)));
+/// assert_eq!(import.description(), &description);
+/// ```
+///
+/// ## Global
+/// ```rust
+/// use wasm_ast::{Import, ImportDescription, Name, GlobalType, ValueType};
+///
+/// let module = "system";
+/// let name = "counter";
+/// let kind = GlobalType::mutable(ValueType::I64);
+/// let description = ImportDescription::Global(kind.clone());
+/// let import = Import::new(module.into(), name.into(), description.clone());
+///
+/// assert_eq!(import, Import::global(module.into(), name.into(), kind));
+/// assert_eq!(import.module(), &Name::new(String::from(module)));
+/// assert_eq!(import.name(), &Name::new(String::from(name)));
+/// assert_eq!(import.description(), &description);
+/// ```
 #[derive(Clone, Debug, Eq, PartialEq)]
 pub struct Import {
     module: Name,
@@ -488,6 +961,7 @@ pub struct Import {
 }
 
 impl Import {
+    /// Creates a new import.
     pub fn new(module: Name, name: Name, description: ImportDescription) -> Self {
         Import {
             module,
@@ -496,14 +970,53 @@ impl Import {
         }
     }
 
+    /// Create a new instance of an `Import` with the given name and description for a table.
+    pub fn table(module: Name, name: Name, table_kind: TableType) -> Self {
+        Import {
+            module,
+            name,
+            description: ImportDescription::Table(table_kind),
+        }
+    }
+
+    /// Create a new instance of an `Import` with the given name and description for a memory.
+    pub fn memory(module: Name, name: Name, memory_kind: MemoryType) -> Self {
+        Import {
+            module,
+            name,
+            description: ImportDescription::Memory(memory_kind),
+        }
+    }
+
+    /// Create a new instance of an `Import` with the given name and description for a function.
+    pub fn function(module: Name, name: Name, function_kind: TypeIndex) -> Self {
+        Import {
+            module,
+            name,
+            description: ImportDescription::Function(function_kind),
+        }
+    }
+
+    /// Create a new instance of an `Import` with the given name and description for a global.
+    pub fn global(module: Name, name: Name, global_kind: GlobalType) -> Self {
+        Import {
+            module,
+            name,
+            description: ImportDescription::Global(global_kind),
+        }
+    }
+
+    /// The name of the module (i.e.m namespace).
     pub fn module(&self) -> &Name {
         &self.module
     }
 
+    /// The name of the import.
     pub fn name(&self) -> &Name {
         &self.name
     }
 
+    /// The description of the import.
     pub fn description(&self) -> &ImportDescription {
         &self.description
     }
@@ -617,29 +1130,6 @@ mod tests {
         assert_eq!(function.body(), &body);
     }
 
-    #[test]
-    fn new_elements() {
-        let kind = ReferenceType::Function;
-        let mode = ElementMode::Active(0, Expression::new(Vec::new()));
-        let initializers = ElementInitializer::FunctionIndex(vec![1]);
-        let element = Element::new(kind, mode.clone(), initializers.clone());
-
-        assert_eq!(element.mode(), &mode);
-        assert_eq!(element.initializers(), &initializers);
-        assert_eq!(element.kind(), &kind);
-    }
-
-    #[test]
-    fn new_data() {
-        let mode = DataMode::Active(0, Expression::new(Vec::new()));
-        let initializer = vec![42];
-        let data = Data::new(mode.clone(), initializer.clone());
-
-        assert_eq!(data.mode(), &mode);
-        assert_eq!(data.initializer(), &initializer);
-        assert_eq!(data.len(), initializer.len());
-    }
-
     #[test]
     fn new_table() {
         let kind = TableType::new(Limit::new(0, None), ReferenceType::Function);
@@ -647,53 +1137,4 @@ mod tests {
 
         assert_eq!(table.kind(), &kind);
     }
-
-    #[test]
-    fn new_memory() {
-        let kind = MemoryType::from(Limit::new(0, None));
-        let memory = Memory::new(kind);
-
-        assert_eq!(memory.kind(), &kind);
-    }
-
-    #[test]
-    fn new_import() {
-        let module = Name::from("test");
-        let name = Name::from("foobar");
-        let kind = MemoryType::from(Limit::new(0, None));
-        let description = ImportDescription::Memory(kind);
-        let import = Import::new(module.clone(), name.clone(), description);
-
-        assert_eq!(import.module(), &module);
-        assert_eq!(import.name(), &name);
-        assert_eq!(import.description(), &description);
-    }
-
-    #[test]
-    fn new_export() {
-        let name = Name::from("foobar");
-        let description = ExportDescription::Function(42);
-        let export = Export::new(name.clone(), description);
-
-        assert_eq!(export.name(), &name);
-        assert_eq!(export.description(), &description);
-    }
-
-    #[test]
-    fn new_start() {
-        let function = 42;
-        let start = Start::new(function);
-
-        assert_eq!(start.function(), function);
-    }
-
-    #[test]
-    fn new_global() {
-        let kind = GlobalType::mutable(IntegerType::I64.into());
-        let expression = Expression::new(Vec::new());
-        let global = Global::new(kind, expression.clone());
-
-        assert_eq!(global.kind(), &kind);
-        assert_eq!(global.initializer(), &expression);
-    }
 }
